package com.hanbit.comm.util;

import java.io.*;


public class Excel {
    // Private Data Definition
    private int  position = 0; //  Ͱ  ִ Buff offset
    private static int XLS_ROW_MAX = 65536; // Max Row length for Excel
    private static int XLS_COL_MAX = 256;   // Max Column length for Excel
    private static int XLS_STR_MAX = 255;   // Max String length for Excel
    
    private int dimRowMin  = XLS_ROW_MAX + 1;
    private int dimColMin  = XLS_COL_MAX + 1;
    private int dimRowMax  = 0;
    private int dimColMax  = 0;

    private OutputStream out = null;

    public Excel(OutputStream out)
    throws IOException{
        this.out = out;
        writeBOF();
    }
    
    public void endExcel()
    throws IOException {
        writeEOF();
        out.flush();
    }

    public void write(int row,int col,String str) 
    throws IOException {
        Double number = null;
        try {
            number = new Double(str);
            writeNumber(row,col,number.doubleValue());
        } catch(NumberFormatException e){
            writeString(row,col,str);
        }
    }
    
    public void writeNumber(int row,int col,double data)
    throws IOException {
        char name   = 0x0203;
        char length = 0x000E;
        char xf     = 0x0000;
        
        if(row >= this.XLS_ROW_MAX) throw new IOException("Exceed Max Excel Row");
        if(col >= this.XLS_COL_MAX) throw new IOException("Exceed Max Excel Column");

        if(row < this.dimRowMin) this.dimRowMin = row;
        if(row > this.dimRowMax) this.dimRowMax = row;

        if(col < this.dimColMin) this.dimColMin = col;
        if(col > this.dimColMax) this.dimColMax = col;
        
        writeChar(name);
        writeChar(length);
        writeChar((char)row);
        writeChar((char)col);
        writeChar((char)xf);
        writeDouble(data);
    }
    
    public void writeString(int row,int col,String data) 
    throws IOException {
        char name     = 0x0204;
        char length = 0x0008;
        char xf     = 0x0000;
        byte littleEndian[] = null;
        if((data == null) || data.trim().equals("")) return;
        
        littleEndian = data.getBytes();
        int  strlen     = littleEndian.length;
             length     = (char)((int)length + strlen);


        if(row >= this.XLS_ROW_MAX) throw new IOException("Exceed Max Excel Row");
        if(col >= this.XLS_COL_MAX) throw new IOException("Exceed Max Excel Column");

        if(row < this.dimRowMin) this.dimRowMin = row;
        if(row > this.dimRowMax) this.dimRowMax = row;

        if(col < this.dimColMin) this.dimColMin = col;
        if(col > this.dimColMax) this.dimColMax = col;
        
        if(littleEndian.length > this.XLS_STR_MAX) {
            strlen += littleEndian.length;
        }
        
        writeChar(name);
        writeChar(length);

        writeChar((char)row);
        writeChar((char)col);
        writeChar(xf);
        writeChar((char)littleEndian.length);

        write(littleEndian);
    }

    private void writeBOF()
    throws IOException {
        char name    = 0x0809;
        char length  = 0x0008;
        
        char version = 0x0005;
        char type    = 0x0010;
        char build   = 0000;
        char year    = 0000;

        writeChar(name);
        writeChar(length);

        writeChar(version);
        writeChar(type);
        writeChar(build);
        writeChar(year);
    }

    private void writeDimension(int row_min,int row_max,
                        int col_min,int col_max)
    throws IOException {
        char name     = 0x0000;
        char length   = 0x000A;
        char reserved = 0x0000;
        int rowMin    = row_min;
        int rowMax    = row_max + 1;
        int colMin    = col_min;
        int colMax    = col_max + 1;
        
        writeChar(name);
        writeChar(length);

        writeChar((char)rowMin);
        writeChar((char)rowMax);
        writeChar((char)colMin);
        writeChar((char)colMax);
        writeChar(reserved);
    }

    private void writeEOF()
    throws IOException {
        char name   =     0x000A;
        char length =     0x0000;
        int rowMin  = this.dimRowMin;
        int rowMax  = this.dimRowMax;
        int colMin  = this.dimColMin;
        int colMax  = this.dimColMax;
        
        if(rowMin == this.XLS_ROW_MAX + 1) rowMin = 0;
        if(colMin == this.XLS_COL_MAX + 1) colMin = 0;
        
        writeChar(name);
        writeChar(length);

        writeDimension(rowMin,rowMax,colMin,colMax);
    }
    
    private void writeChar(char bigEndian)
    throws IOException {
        byte littleEndian[] = new byte[2];

        // Convert Little Endian to Big Endian
        littleEndian[0] = (byte) ((bigEndian & 0x00ff)) ;
        littleEndian[1] = (byte) ((bigEndian & 0xff00) >> 8) ;
        write(littleEndian);
    }

    private void writeDouble(double bigEndian)
    throws IOException {
        byte littleEndian[] = new byte[8];
        long l = Double.doubleToLongBits(bigEndian);
        littleEndian[7] = (byte)((l & 0xFF00000000000000L )>>56);
        littleEndian[6] = (byte)((l & 0x00FF000000000000L )>>48 );
        littleEndian[5] = (byte)((l & 0x0000FF0000000000L )>>40 );
        littleEndian[4] = (byte)((l & 0x000000FF00000000L )>>32 );
        littleEndian[3] = (byte)((l & 0x00000000FF000000L )>>24 );
        littleEndian[2] = (byte)((l & 0x0000000000FF0000L )>>16 );
        littleEndian[1] = (byte)((l & 0x000000000000FF00L )>>8 );
        littleEndian[0] = (byte)((l & 0x00000000000000FFL ) );
        write(littleEndian);
    }
    
    private void write(byte[] data)
    throws IOException {
        int size = 0;
        if(data == null) return;
        out.write(data);
    }
}
